local _, S = ...
local pairs, ipairs, string, type, time = pairs, ipairs, string, type, time

S.Category = {}

-- Table of attributes and their filter methods
S.Category.attributesTable = {
    --[[
    ["NAME"] = {
        ["name"] = NAME,
        ["attribute"] = "name",
        ["type"] = "STRINGS"
    },
    ]]
    ["TYPE"] = {
        ["name"] = TYPE,
        ["type"] = "VALUES",
        ["values"] = {
            -- Table is built later, scroll down
        },
        ["func"] = function(value, itemData)
            return itemData.classID == value[1] and itemData.subClassID == value[2]
        end
    },
    ["EQUIP_LOCATION"] = {
        ["name"] = S.Localize("FILTER_EQUIP_LOCATION"),
        ["type"] = "VALUES",
        ["values"] = {
            {
                ["name"] = S.Localize("SUBFILTER_EQUIP_LOCATION_HEAD"),
                ["value"] = {"INVTYPE_HEAD"}
            },
            {
                ["name"] = S.Localize("SUBFILTER_EQUIP_LOCATION_NECK"),
                ["value"] = {"INVTYPE_NECK"}
            },
            {
                ["name"] = S.Localize("SUBFILTER_EQUIP_LOCATION_SHOULDER"),
                ["value"] = {"INVTYPE_SHOULDER"}
            },
            {
                ["name"] = S.Localize("SUBFILTER_EQUIP_LOCATION_BACK"),
                ["value"] = {"INVTYPE_CLOAK"}
            },
            {
                ["name"] = S.Localize("SUBFILTER_EQUIP_LOCATION_CHEST"),
                ["value"] = {"INVTYPE_CHEST","INVTYPE_ROBE"}
            },
            {
                ["name"] = S.Localize("SUBFILTER_EQUIP_LOCATION_SHIRT"),
                ["value"] = {"INVTYPE_BODY"}
            },
            {
                ["name"] = S.Localize("SUBFILTER_EQUIP_LOCATION_TABARD"),
                ["value"] = {"INVTYPE_TABARD"}
            },
            {
                ["name"] = S.Localize("SUBFILTER_EQUIP_LOCATION_WRIST"),
                ["value"] = {"INVTYPE_WRIST"}
            },
            {
                ["name"] = S.Localize("SUBFILTER_EQUIP_LOCATION_HANDS"),
                ["value"] = {"INVTYPE_HAND"}
            },
            {
                ["name"] = S.Localize("SUBFILTER_EQUIP_LOCATION_WAIST"),
                ["value"] = {"INVTYPE_WAIST"}
            },
            {
                ["name"] = S.Localize("SUBFILTER_EQUIP_LOCATION_LEGS"),
                ["value"] = {"INVTYPE_LEGS"}
            },
            {
                ["name"] = S.Localize("SUBFILTER_EQUIP_LOCATION_FEET"),
                ["value"] = {"INVTYPE_FEET"}
            },
            {
                ["name"] = S.Localize("SUBFILTER_EQUIP_LOCATION_FINGER"),
                ["value"] = {"INVTYPE_FINGER"}
            },
            {
                ["name"] = S.Localize("SUBFILTER_EQUIP_LOCATION_TRINKET"),
                ["value"] = {"INVTYPE_TRINKET"}
            },
            {
                ["name"] = S.Localize("SUBFILTER_EQUIP_LOCATION_OFF_HAND"),
                ["value"] = {"INVTYPE_SHIELD","INVTYPE_HOLDABLE"}
            },
            {
                ["name"] = S.Localize("SUBFILTER_EQUIP_LOCATION_RELIC"),
                ["value"] = {"INVTYPE_RELIC"}
            },
        },
        ["func"] = function(value, itemData)
            for _, v in pairs(value) do
                if itemData.equipLoc == v then
                    return true
                end
            end
            return false
        end
    },
    --[[
    ["STATS"] = {
        ["name"] = S.Localize("FILTER_STATS"),
        ["type"] = "RANGES",
        ["ranges"] = {

        }
    },
    ]]
    ["QUALITY"] = {
        ["name"] = S.Localize("FILTER_QUALITY"),
        ["type"] = "VALUES",
        ["values"] = {
            -- Table is built later, scroll down
        },
        ["func"] = function(value, itemData)
            print(value)
            return itemData.quality == value
        end
    },
    --[[
    ["TIME_ADDED"] = {
        ["name"] = S.Localize("FILTER_TIME_ADDED"),
        ["type"] = "RANGES"
    },
    ["BINDING"] = {
        ["name"] = S.Localize("FILTER_BINDING"),
        ["type"] = "VALUES"
    },
    ["EXPANSION"] = {
        ["name"] = S.Localize("FILTER_EXPANSION"),
        ["type"] = "VALUES"
    },
    ["LEVEL"] = {
        ["name"] = S.Localize("FILTER_LEVEL"),
        ["type"] = "RANGES"
    },
    ["MARKER_ICON"] = {
        ["name"] = S.Localize("FILTER_MARKER_ICON"),
        ["type"] = "VALUES"
    },
    ]]
}
-- Build the value table for item types
for classKey, classID in pairs(Enum.ItemClass) do
    local className = GetItemClassInfo(classID)
    if className and not string.find(string.lower(className), "obsolete") then
        local foundLast = false
        local subClassID = 0
        while not foundLast do
            local subClassName = GetItemSubClassInfo(classID, subClassID)
            if type(subClassName) == "string" and #subClassName > 0 then
                if not string.find(string.lower(subClassName), "obsolete") then
                    S.Category.attributesTable["TYPE"].values[classID * 1000 + subClassID] = {
                        ["heading"] = className,
                        ["name"] = subClassName,
                        ["value"] = {classID, subClassID}
                    }
                end
            elseif subClassID >= 1 then
                foundLast = true
            end
            subClassID = subClassID + 1
        end
    end
end
-- Build the table for item qualities
local qualityID = 0
local qualityName = S.Utils.GetItemQualityName(qualityID)
while qualityName do
    S.Category.attributesTable["QUALITY"].values[qualityID] = {
        ["name"] = qualityName,
        ["value"] = qualityID
    }

    qualityID = qualityID + 1
    qualityName = S.Utils.GetItemQualityName(qualityID)
end


local function Data(id, guid)
    return S.Settings.Get("categories2", guid)[id]
end

function S.Category.GetName(id, guid)
    return Data(id, guid).name
end
function S.Category.SetName(id, name, guid)
    Data(id, guid).name = name
    S.Utils.TriggerEvent("SettingChanged-categories2")
end
function S.Category.GetIconTexture(id, guid)
    return "INTERFACE\\ICONS\\"..Data(id, guid).icon
end
function S.Category.SetIcon(id, icon, guid)
    Data(id, guid).icon = icon
    S.Utils.TriggerEvent("SettingChanged-categories2")
end

function S.Category.AddNew()
    local data = S.Settings.Get("categories2", guid)
    local id = #data + 1
    if id <= S.Utils.GetMaxNumCategories() then
        data[id] = {
            ["name"] = S.Localize("CONFIG_CATEGORIES_DEFAULT_NAME"),
            ["icon"] = "INV_MISC_QUESTIONMARK",
            ["attributes"] = {}
        }
        S.Utils.TriggerEvent("SettingChanged-categories2")
    end
end

function S.Category.Delete(id)
    local data = S.Settings.Get("categories2", guid)
    for i = id, #data do
        data[i] = data[i + 1]
    end
    S.Utils.TriggerEvent("SettingChanged-categories2")
end

function S.Category.GetAttribute(id, attributeKey, guid)
    local data = Data(id, guid).attributes[attributeKey]
    if data then 
        return data
    else
        return {}
    end
end

function S.Category.ToggleAttributeValue(id, attributeKey, valueKey, guid)
    local data = Data(id, guid).attributes
    if not data[attributeKey] then
        data[attributeKey] = {}
    end
    data[attributeKey][valueKey] = not data[attributeKey][valueKey]
    
    -- Delete the table if all values are false
    local attributeHasData = false
    for k,v in pairs(data[attributeKey]) do
        if v then
            attributeHasData = true
        end
    end
    if not attributeHasData then
        data[attributeKey] = nil
    end

    S.Utils.TriggerEvent("SettingChanged-categories2")
    S.Utils.TriggerEvent("CategorySelected")
end

function S.Category.Filter(id, itemData, guid)
    local data = Data(id, guid).attributes

    for attributeKey, attribute in pairs(data) do
        local filtered = false
        local func = S.Category.attributesTable[attributeKey].func
        local values = S.Category.attributesTable[attributeKey].values

        for value, enabled in pairs(attribute) do
            if enabled then
                if func(values[value].value, itemData) then
                    filtered = true
                end
            end
        end

        if not filtered then
            return true
        end
    end

    return false
end