local _, S = ...
local pairs, ipairs, string, type, time = pairs, ipairs, string, type, time

-- Settings frame is built using data from this table
local settingsTable = {
    [1] = {
        ["name"] = S.Localize("CONFIG_APPEARANCE_SKINNING"),
        ["left"] = {
            {
                ["name"] = S.Localize("CONFIG_APPEARANCE_SKINNING"),
                ["type"] = "RADIO",
                ["setting"] = "skinning",
                ["buttons"] = {
                    {
                        ["name"] = S.Localize("CONFIG_APPEARANCE_SKINNING_DEFAULT"),
                        ["value"] = S.Skinning.DEFAULT
                    },
                    {
                        ["name"] = S.Localize("CONFIG_APPEARANCE_SKINNING_CLEAN"),
                        ["value"] = S.Skinning.CLEAN
                    },
                    {
                        ["name"] = S.Localize("CONFIG_APPEARANCE_SKINNING_ADDONSKINS"),
                        ["value"] = S.Skinning.ADDONSKINS,
                        ["Enabled"] = function() return S.Skinning.AddOnSkinsAvailable() end
                    }
                }
            },
            {
                ["name"] = S.Localize("CONFIG_APPEARANCE_SCALE"),
                ["type"] = "SLIDER",
                ["setting"] = "scale",
                ["min"] = 0.5,
                ["max"] = 1.5,
                ["precision"] = 2,
            },
            {
                ["type"] = "SPACER"
            },
            {
                ["name"] = S.Localize("CONFIG_APPEARANCE_BACKDROP"),
                ["type"] = "DROPDOWN",
                ["setting"] = "backdrop",
                ["BuildDropdown"] = function()
                    local function OnClick(self)
                        S.Settings.Set("backdrop", self.data1)
                    end
                    local function OnEnter(self)
                        S.FrameTools.UpdateBackdrops(self.data1)
                    end
                    local function OnLeave(self)
                        S.FrameTools.UpdateBackdrops()
                    end
                    local backdrops = S.Utils.GetBackgrounds()
                    for k, backdrop in pairs(backdrops) do
                        S.Dropdown.AddEntry(backdrop, OnClick, backdrop)
                        S.Dropdown.OnEnter(OnEnter)
                        S.Dropdown.OnLeave(OnLeave)
                    end
                end
            },
            {
                ["name"] = COLOR,
                ["type"] = "COLOR",
                ["setting"] = "backdropColor",
            },
        },
        ["right"] = {
            {
                ["name"] = S.Localize("CONFIG_APPEARANCE_FONT"),
                ["type"] = "DROPDOWN",
                ["setting"] = "font",
                ["BuildDropdown"] = function()
                    local function OnClick(self)
                        S.Settings.Set("font", self.data1)
                    end
                    local fonts = S.Utils.GetFonts()
                    for k, font in pairs(fonts) do
                        S.Dropdown.AddEntry(font, OnClick, font)
                        S.Dropdown.SetFont(font)
                    end
                end
            },
            {
                ["name"] = S.Localize("CONFIG_APPEARANCE_FONT_SIZE"),
                ["type"] = "SLIDER",
                ["setting"] = "fontSizePts",
                ["min"] = 6,
                ["max"] = 18,
                ["precision"] = 1,
            },
            {
                ["name"] = S.Localize("CONFIG_APPEARANCE_FONT_OUTLINE"),
                ["type"] = "SLIDER",
                ["setting"] = "fontOutline",
                ["min"] = 0,
                ["max"] = 2,
                ["precision"] = 0,
                ["values"] = {
                    [0] = S.Localize("CONFIG_APPEARANCE_FONT_OUTLINE_1"),
                    [1] = S.Localize("CONFIG_APPEARANCE_FONT_OUTLINE_2"),
                    [2] = S.Localize("CONFIG_APPEARANCE_FONT_OUTLINE_3")
                }
            },
            {
                ["name"] = S.Localize("CONFIG_APPEARANCE_FONT_SHADOW"),
                ["type"] = "SLIDER",
                ["setting"] = "fontShadow",
                ["min"] = 0,
                ["max"] = 4,
                ["precision"] = 0
            },
        }
    },
    [2] = {
        ["name"] = S.Localize("CONFIG_FEATURES"),
        ["left"] = {
            {
                ["name"] = S.Localize("CONFIG_BEHAVIOR_EXTENDED_TOOLTIPS"),
                ["type"] = "CHECKBOX",
                ["setting"] = "tooltipInfo"
            },
            {
                ["name"] = S.Localize("CONFIG_BEHAVIOR_COMBINE_STACKS"),
                ["type"] = "CHECKBOX",
                ["setting"] = "combineStacks"
            },
        },
        ["right"] = {
            
        }
    },
    [3] = {
        ["name"] = S.Localize("CONFIG_LIST"),
        ["left"] = {
            {
                ["name"] = S.Localize("CONFIG_APPEARANCE_ICON_SIZE"),
                ["type"] = "SLIDER",
                ["setting"] = "iconSize",
                ["min"] = 10,
                ["max"] = 50,
                ["precision"] = 1,
            },
            {
                ["name"] = S.Localize("CONFIG_APPEARANCE_PADDING"),
                ["type"] = "SLIDER",
                ["setting"] = "padding",
                ["min"] = 0,
                ["max"] = 20,
                ["precision"] = 1,
            },
            {
                ["name"] = S.Localize("CONFIG_APPEARANCE_ICON_BORDERS"),
                ["type"] = "CHECKBOX",
                ["setting"] = "iconBorders"
            },
            {
                ["name"] = S.Localize("CONFIG_APPEARANCE_ICON_BORDER_THICKNESS"),
                ["type"] = "SLIDER",
                ["setting"] = "iconBorderThickness",
                ["min"] = 0,
                ["max"] = 10,
                ["precision"] = 2,
            },
            {
                ["name"] = S.Localize("CONFIG_APPEARANCE_ICON_SHAPE"),
                ["type"] = "CHECKBOX",
                ["setting"] = "iconShape"
            },
        },
        ["right"] = {
            
        }
    },
    [4] = {
        ["name"] = S.Localize("CONFIG_GRID"),
        ["left"] = {
            {
                ["name"] = S.Localize("CONFIG_APPEARANCE_ICON_SIZE"),
                ["type"] = "SLIDER",
                ["setting"] = "iconSizeGrid",
                ["min"] = 25,
                ["max"] = 50,
                ["precision"] = 1,
            },
            {
                ["name"] = S.Localize("CONFIG_APPEARANCE_PADDING"),
                ["type"] = "SLIDER",
                ["setting"] = "paddingGrid",
                ["min"] = 0,
                ["max"] = 20,
                ["precision"] = 1,
            },
            {
                ["name"] = S.Localize("CONFIG_APPEARANCE_ICON_BORDERS"),
                ["type"] = "CHECKBOX",
                ["setting"] = "iconBordersGrid"
            },
            {
                ["name"] = S.Localize("CONFIG_APPEARANCE_ICON_BORDER_THICKNESS"),
                ["type"] = "SLIDER",
                ["setting"] = "iconBorderThicknessGrid",
                ["min"] = 0,
                ["max"] = 10,
                ["precision"] = 2,
            },
            {
                ["name"] = S.Localize("CONFIG_APPEARANCE_ICON_SHAPE"),
                ["type"] = "CHECKBOX",
                ["setting"] = "iconShapeGrid"
            },
        },
        ["right"] = {
            
        }
    },
    [5] = {
        ["name"] = S.Localize("CONFIG_AUTOMATION"),
        ["left"] = {
            {
                ["name"] = "|cffff0522NOT YET IMPLEMENTED",
                ["type"] = "SPACER"
            },
        },
        ["right"] = {
            
        }
    },
    --[[[6] = {
        ["name"] = S.Localize("CONFIG_CATEGORIES"),
        ["left"] = {
            {
                ["name"] = "|cffff0522NOT YET IMPLEMENTED",
                ["type"] = "SPACER"
            },
        },
        ["right"] = {
            
        }
    },]]
    --[[[7] = {
        ["name"] = S.Localize("CONFIG_PROFILES"),
        ["left"] = {

        },
        ["right"] = {
            
        }
    }]]
}


-- Slider
local function RoundToDecimalPlaces(value, numPlaces)
    print(math.floor(value * 10^numPlaces) / 10^numPlaces)
    return math.floor(value * 10^numPlaces) / 10^numPlaces
end
local function SetSliderText(self)
    if self.values then
        self.valueString:SetText(self.values[self:GetValue()])
    else
        self.valueString:SetText(string.format("%."..self.precision.."f", self:GetValue()))
    end
end
local function SliderOnSettingChanged(self, event, value)
    SetSliderText(self)
end
local function OnSliderValueChanged(self)
    SetSliderText(self)
    --S.Settings.Set(self.setting, self:GetValue())
end
local function OnSliderMouseUp(self)
    if self.values then
        self:SetValue(math.floor(self:GetValue() + 0.5))
    end
    S.Settings.Set(self.setting, self:GetValue())
end
local function CreateSlider(parent, name, setting, min, max, precision, values)
    local f = CreateFrame("FRAME", nil, parent)
    f:SetSize(224, 64)

    local slider = CreateFrame("SLIDER", nil, f)
    slider.setting = setting
    slider.precision = precision
    slider:SetValueStep(10^(-precision))
    slider:SetObeyStepOnDrag(true)
    slider:SetSize(208, 56)
    slider:SetPoint("BOTTOM", -16, -8)
    slider:SetOrientation("HORIZONTAL")
    slider:SetScript("OnValueChanged", OnSliderValueChanged)
    slider:SetScript("OnMouseUp", OnSliderMouseUp)
    slider:SetMinMaxValues(min, max)
    S.Utils.RunOnEvent(slider, "SettingChanged-"..setting, SliderOnSettingChanged)

    if values then
        slider.values = values
    end

    slider:SetThumbTexture("Interface\\Addons\\Sorted\\Textures\\Slider-Thumb")
    slider:GetThumbTexture():SetSize(40, 40)

    slider.rail = slider:CreateTexture(nil, "BACKGROUND")
    slider.rail:SetAllPoints()
    slider.rail:SetTexture("Interface\\Addons\\Sorted\\Textures\\Slider-Rail")

    slider.nameString = slider:CreateFontString(nil, "OVERLAY", "SortedFont")
    slider.nameString:SetTextColor(S.Utils.GetButtonTextColor():GetRGB())
    slider.nameString:SetPoint("BOTTOM", slider, "TOP", 16, -16)
    slider.nameString:SetText(name)
    slider.nameString:SetTextScale(1.2)

    slider.valueString = slider:CreateFontString(nil, "OVERLAY", "SortedFont")
    slider.valueString:SetTextColor(1, 1, 1)
    slider.valueString:SetPoint("LEFT", slider, "RIGHT", -4, 0)

    slider:SetScript("OnShow", function(self) self:SetValue(S.Settings.Get(self.setting)) end)
    return f
end


-- Checkbox
local function CheckboxOnSettingChanged(self, event, value)
    self:SetChecked(value == 1)
end
local function OnCheckboxClicked(self)
    if self:GetChecked() then
        S.Settings.Set(self.setting, 1)
    else
        S.Settings.Set(self.setting, 0)
    end
end
local function CreateCheckbox(parent, name, setting)
    local f = CreateFrame("FRAME", nil, parent)
    f:SetSize(224, 64)
    local cb = CreateFrame("CheckButton", nil, f)
    cb.setting = setting
    cb:SetSize(32, 32)
    cb:SetNormalTexture("Interface\\Addons\\Sorted\\Textures\\Checkbox")
    cb:SetHighlightTexture("Interface\\Addons\\Sorted\\Textures\\Checkbox-Highlight")
    cb:SetPushedTexture("Interface\\Addons\\Sorted\\Textures\\Checkbox")
    cb:SetCheckedTexture("Interface\\Addons\\Sorted\\Textures\\Checkbox-Tick")
    cb:SetPoint("LEFT", f, 2, 0)
    cb:SetScript("OnClick", OnCheckboxClicked)
    S.Utils.RunOnEvent(cb, "SettingChanged-"..setting, CheckboxOnSettingChanged)
    f.checkButton = cb
    f.nameString = f:CreateFontString(nil, "OVERLAY", "SortedFont")
    f.nameString:SetText(name)
    f.nameString:SetPoint("LEFT", cb, "RIGHT", 8, 0)
    f.nameString:SetTextColor(1, 1, 1)
    f.nameString:SetTextScale(1.2)

    cb:SetScript("OnShow", function(self) cb:SetChecked(S.Settings.Get(self.setting) == 1) end)
    return f
end


-- Radio buttons
local function RadioButtonOnSettingChanged(self, event, value)
    self:SetChecked(value == self.value)
end
local function OnRadioButtonClicked(self)
    for k,v in pairs(self.parent.buttons) do
        v:SetChecked(v == self)
    end
    S.Settings.Set(self.setting, self.value)
end
local function CreateRadio(parent, name, setting, buttons)
    local f = CreateFrame("FRAME", nil, parent)
    f:SetSize(224, 24 + 40 * #buttons)
    f.nameString = f:CreateFontString(nil, "OVERLAY", "SortedFont")
    f.nameString:SetText(name)
    f.nameString:SetPoint("TOP", 0, -4)
    f.nameString:SetTextColor(S.Utils.GetButtonTextColor():GetRGB())
    f.nameString:SetTextScale(1.3)
    f.buttons = {}
    for i,v in ipairs(buttons) do
        local cb = CreateFrame("CheckButton", nil, f)
        cb:SetNormalTexture("Interface\\Addons\\Sorted\\Textures\\Radio-Button")
        cb:SetHighlightTexture("Interface\\Addons\\Sorted\\Textures\\Radio-Button-Highlight")
        cb:SetPushedTexture("Interface\\Addons\\Sorted\\Textures\\Radio-Button")
        cb:SetCheckedTexture("Interface\\Addons\\Sorted\\Textures\\Radio-Button-Dot")
        cb:SetPoint("TOPLEFT", f, "TOPLEFT", 0, -24 - 40 * (i - 1))
        cb:SetSize(32, 32)
        cb.parent = f
        cb.setting = setting
        cb.value = v.value
        if v.Enabled then
            cb.ShouldEnable = v.Enabled
            cb:SetScript("OnShow", function(self)
                if self.ShouldEnable() then
                    self.nameString:SetTextColor(1, 1, 1)
                    self:Enable()
                else
                    self.nameString:SetTextColor(0.5, 0.5, 0.5)
                    self:Disable()
                end
            end)
        end
        cb:SetScript("OnClick", OnRadioButtonClicked)
        S.Utils.RunOnEvent(cb, "SettingChanged-"..setting, RadioButtonOnSettingChanged)
        cb.nameString = f:CreateFontString(nil, "OVERLAY", "SortedFont")
        cb.nameString:SetText(v.name)
        cb.nameString:SetPoint("LEFT", cb, "RIGHT", 4, 0)
        cb.nameString:SetTextColor(1, 1, 1)
        cb.nameString:SetTextScale(1.2)
        f.buttons[i] = cb
        cb:HookScript("OnShow", function(self) 
            self:SetChecked(S.Settings.Get(self.setting) == self.value) 
        end)
    end
    return f
end


-- Dropdown
local function DropdownOnSettingChanged(self, event, value)
    self.text:SetText(value)
end
local function CreateDropdown(parent, name, setting, BuildDropdown)
    local f = S.FrameTools.CreateDropdown(parent, name, BuildDropdown)
    f.button.setting = setting
    f.button:SetScript("OnShow", function(self)
        self.text:SetText(S.Settings.Get(self.setting))
    end)
    S.Utils.RunOnEvent(f.button, "SettingChanged-"..setting, DropdownOnSettingChanged)
    return f
end


-- Color
local function ColorButtonOnSettingChanged(self, event, value)
    self:GetNormalTexture():SetVertexColor(unpack(S.Settings.Get(self.setting)))
end
local function ColorButtonCancelCallback()
    S.Settings.Set(ColorPickerFrame.setting, ColorPickerFrame.previousColor)
end
local function ColorButtonCallback(restore)
    local r,g,b,a
    if restore then
        r,g,b,a = unpack(restore)
    else
        r,g,b = ColorPickerFrame:GetColorRGB()
        a = 1 - OpacitySliderFrame:GetValue()
    end
    S.Settings.Set(ColorPickerFrame.setting, {r,g,b,a})
end
local function OnColorButtonClick(self)
    local color = S.Settings.Get(self.setting)
    ColorPickerFrame:SetColorRGB(unpack(color))
    ColorPickerFrame.opacity = 1 - color[4]
    ColorPickerFrame.hasOpacity = true
    ColorPickerFrame.previousColor = color
    ColorPickerFrame.func = ColorButtonCallback
    ColorPickerFrame.opacityFunc = ColorButtonCallback
    ColorPickerFrame.cancelFunc = ColorButtonCancelCallback
    ColorPickerFrame.setting = self.setting
    ColorPickerFrame:Show()
end
local function CreateColor(parent, name, setting)
    local f = CreateFrame("FRAME", nil, parent)
    f:SetSize(224, 64)

    f.colorButton = CreateFrame("BUTTON", nil, f)
    f.colorButton:SetSize(32, 32)
    f.colorButton:SetPoint("LEFT", 16, 0)
    f.colorButton.bg = f.colorButton:CreateTexture(nil, "BACKGROUND")
    f.colorButton.bg:SetTexture("Interface\\Addons\\Sorted\\Textures\\Radio-Button")
    f.colorButton.bg:SetAllPoints()
    f.colorButton:SetNormalTexture("Interface\\Addons\\Sorted\\Textures\\Radio-Button-Color-Dot")
    f.colorButton:SetHighlightTexture("Interface\\Addons\\Sorted\\Textures\\Radio-Button-Highlight")
    f.colorButton:SetScript("OnClick", OnColorButtonClick)
    f.colorButton.setting = setting
    S.Utils.RunOnEvent(f.colorButton, "SettingChanged-"..setting, ColorButtonOnSettingChanged)

    f.nameString = f:CreateFontString(nil, "OVERLAY", "SortedFont")
    f.nameString:SetText(name)
    f.nameString:SetPoint("LEFT", f.colorButton, "RIGHT", 4, 0)
    f.nameString:SetTextColor(1, 1, 1)
    f.nameString:SetTextScale(1.2)

    return f
end


-- EditBox
local function EditBoxOnSettingChanged(self, event, value)
    self:SetText(value)
end
local function OnEditBoxTextChanged(self)
    S.Settings.Set(self.setting, self:GetText())
end
local function CreateEditBox(parent, name, setting, instruction)
    local f = S.FrameTools.CreateEditBox(parent, name, instruction)

    f.editBox.setting = setting
    f.editBox:HookScript("OnTextChanged", OnEditBoxTextChanged)
    f.editBox:SetScript("OnEnterPressed", f.editBox.ClearFocus)
    f.editBox:HookScript("OnShow", function(self)
        self:SetText(S.Settings.Get(self.setting))
    end)
    S.Utils.RunOnEvent(f.editBox, "SettingChanged-"..setting, EditBoxOnSettingChanged)
    return f
end


-- Spacer
local function CreateSpacer(parent, name)
    local f = CreateFrame("FRAME", nil, parent)
    f:SetSize(224, 48)
    if name then
        f.nameString = f:CreateFontString(nil, "OVERLAY", "SortedFont")
        f.nameString:SetText(name)
        f.nameString:SetPoint("CENTER")
        f.nameString:SetTextColor(S.Utils.GetButtonTextColor():GetRGB())
        f.nameString:SetTextScale(1.2)
    end
    return f
end



-- BUILD SETTINGS FROM TABLE
local function CreateSettingWidget(parent, widgetData)
    local widget
    if not parent.y then
        parent.y = -16
    end
    if widgetData.type == "SLIDER" then
        widget = CreateSlider(parent, widgetData.name, widgetData.setting, widgetData.min, widgetData.max, widgetData.precision, widgetData.values)
    elseif widgetData.type == "CHECKBOX" then
        widget = CreateCheckbox(parent, widgetData.name, widgetData.setting)
    elseif widgetData.type == "RADIO" then
        widget = CreateRadio(parent, widgetData.name, widgetData.setting, widgetData.buttons)
    elseif widgetData.type == "DROPDOWN" then
        widget = CreateDropdown(parent, widgetData.name, widgetData.setting, widgetData.BuildDropdown)
    elseif widgetData.type == "COLOR" then
        widget = CreateColor(parent, widgetData.name, widgetData.setting)
    elseif widgetData.type == "SPACER" then
        widget = CreateSpacer(parent, widgetData.name)
    end
    widget:SetPoint("TOP", parent, "TOP", 0, parent.y)
    parent.y = parent.y - widget:GetHeight()
end

local settingsFrames = {}
for k,v in pairs(settingsTable) do
    local f = CreateFrame("FRAME")
    settingsFrames[k] = f
    f.name = settingsTable[k].name
    
    f.left = CreateFrame("FRAME", nil, f)
    f.left:SetPoint("TOPLEFT")
    f.left:SetPoint("BOTTOMRIGHT", f, "BOTTOM", -16, 0)
    for i, widget in ipairs(v.left) do
        CreateSettingWidget(f.left, widget)
    end
    
    f.right = CreateFrame("FRAME", nil, f)
    f.right:SetPoint("TOPLEFT", f, "TOP", -16, 0)
    f.right:SetPoint("BOTTOMRIGHT", -32, 0)
    for i, widget in ipairs(v.right) do
        CreateSettingWidget(f.right, widget)
    end
end


-- CREATE CATEGORIES FRAME (to be built by SettingsFrame_Categories.lua)
local f = CreateFrame("FRAME")
settingsFrames[#settingsFrames + 1] = f
f.name = S.Localize("CONFIG_CATEGORIES")
S.categoriesSettingsFrame = f


-- BUILD PROFILES FRAME
local f = CreateFrame("FRAME")
settingsFrames[#settingsFrames + 1] = f
f.name = S.Localize("CONFIG_PROFILES")
local function ProfileDropdownEntryOnClick(self)
    S.Settings.SetProfile(self.data1)
end
f.dropdown = CreateDropdown(f, S.Localize("CONFIG_PROFILES_PROFILE"), "profileName", function()
    for k, settingsProfile in pairs(Sorted_SettingsProfiles) do
        S.Dropdown.AddEntry(settingsProfile.profileName, ProfileDropdownEntryOnClick, k)
    end
end)
f.dropdown:SetPoint("TOPLEFT", 64, -128)
f.buttonNew = S.FrameTools.CreateBasicTextButton(f, S.Localize("CONFIG_PROFILES_NEW"), function(self)
    S.Settings.CreateNewProfile(nil)
end)
f.buttonCopy = S.FrameTools.CreateBasicTextButton(f, S.Localize("CONFIG_PROFILES_COPY"), S.Settings.CopyProfile)
f.buttonDelete = S.FrameTools.CreateBasicTextButton(f, S.Localize("CONFIG_PROFILES_DELETE"), function(self)
    S.Settings.DeleteProfile()
    if next(Sorted_SettingsProfiles) then
        S.Settings.SetProfile(next(Sorted_SettingsProfiles))
    else
        SortedSettingsFrame:Hide()
        SortedSettingsProfilesFrame:Show()
    end
end)
f.buttonNew:SetPoint("LEFT", f.dropdown.button, "RIGHT", 16, -2)
f.buttonCopy:SetPoint("LEFT", f.buttonNew, "RIGHT")
f.buttonDelete:SetPoint("LEFT", f.buttonCopy, "RIGHT")

f.editBox = CreateEditBox(f, S.Localize("CONFIG_PROFILES_CHANGE_NAME"), "profileName", S.Localize("CONFIG_PROFILES_CHANGE_NAME_INSTRUCTION"))
f.editBox:SetPoint("TOPLEFT", f.dropdown, "BOTTOMLEFT", 0, -64)
f.editBox.editBox:SetSize(384, 16)



-- BUILD MAIN SETTINGS FRAME
f = CreateFrame("FRAME", "SortedSettingsFrame", UIParent)
S.settingsFrame = f
table.insert(UISpecialFrames, "SortedSettingsFrame")
f:SetPoint("CENTER")
f:SetSize(800, 600)
f:SetClampedToScreen(true)
f:EnableMouse()
f:SetMovable(true)
f:SetFrameStrata("HIGH")
f:SetFrameLevel(632)
S.FrameTools.AddOuterShadow(f, 100)
--S.FrameTools.AddSortedBackdrop(f)
f.bg = f:CreateTexture()
f.bg:SetTexture("Interface\\Addons\\Sorted\\Textures\\Abstract", "REPEAT", "REPEAT")
f.bg:SetVertexColor(0.6, 0.6, 0.6)
f.bg:SetDrawLayer("BACKGROUND")
f.bg:SetAllPoints()
f.bg:SetVertTile(true)
f.bg:SetHorizTile(true)
f.closeButton = S.FrameTools.CreateCloseButton(f)
f.closeButton:SetSize(24, 24)
f.closeButton:SetNormalTexture("Interface\\Addons\\Sorted\\Textures\\redbutton2x-Clean")
f.closeButton:SetHighlightTexture("Interface\\Addons\\Sorted\\Textures\\redbutton2x-Clean")
f.closeButton:GetHighlightTexture():SetTexCoord(0.1484375, 0.296875, 0, 0.3125)
f.closeButton:GetHighlightTexture():SetAlpha(0.6)
f.closeButton:SetPushedTexture("Interface\\Addons\\Sorted\\Textures\\redbutton2x-Clean")
 -- Prevent click-through and do dragging
f:SetScript("OnMouseDown", function(self) 
    self:StartMoving()
end)
f:SetScript("OnMouseUp", function(self) 
    self:StopMovingOrSizing()
end)
f:Hide()
f:SetScript("OnShow", function(self)
    if not S.Settings.HasProfile() then
        self:Hide()
        S.settingsProfilesFrame:Show()
    end
end)

f.left = CreateFrame("FRAME", nil, f)
f.left:SetPoint("TOPLEFT")
f.left:SetPoint("BOTTOMRIGHT", f, "BOTTOMLEFT", 192, 0)
f.left.bg = f.left:CreateTexture(nil, "BACKGROUND")
f.left.bg:SetColorTexture(0, 0, 0, 0.4)
f.left.bg:SetAllPoints()
f.left.title = f.left:CreateTexture(nil, "ARTWORK")
f.left.title:SetTexture("Interface\\Addons\\Sorted\\Textures\\Title")
f.left.title:SetPoint("TOPLEFT", 8, -8)
f.left.title:SetSize(192, 48)
f.left.buttons = {}
local selectedTab = 1
function f.left.UpdateButtons()
    for index, b in pairs(f.left.buttons) do
        if index == selectedTab then
            b.frame:Show()
            b.text:SetTextColor(1, 1, 1)
            b:SetNormalTexture("Interface\\Addons\\Sorted\\Textures\\Dropdown-Highlight")
            b:GetNormalTexture():SetVertexColor(0.9, 0.7, 0.04)
            b:GetNormalTexture():SetTexCoord(0, 1, 0.02, 0.98)
            b:GetNormalTexture():SetBlendMode("ADD")
        else
            b.frame:Hide()
            b.text:SetTextColor(S.Utils.GetButtonTextColor():GetRGB())
            b:SetNormalTexture("Interface\\Addons\\Sorted\\Textures\\Transparent")
        end
    end
end
local function OnButtonClick(self)
    selectedTab = self:GetID()
    f.left.UpdateButtons()
end
function f.left.AddButton(index, name)
    local b = CreateFrame("BUTTON", nil, f.left)
    b:SetSize(160, 40)
    b:SetPoint("TOP", 0, -64 - index * 48)
    b:SetID(index)
    b:SetHighlightTexture("Interface\\Addons\\Sorted\\Textures\\Dropdown-Highlight")
    b:GetHighlightTexture():SetVertexColor(0.3, 0.5, 0.8)
    b:GetHighlightTexture():SetTexCoord(0, 1, 0.1, 0.9)
    b.text = b:CreateFontString(nil, "OVERLAY", "SortedFont")
    b.text:SetText(name)
    b.text:SetTextColor(S.Utils.GetButtonTextColor():GetRGB())
    b.text:SetTextScale(1.3)
    b.text:SetPoint("CENTER")
    b:RegisterForClicks("LeftButtonDown")
    b:SetScript("OnClick", OnButtonClick)
    f.left.buttons[index] = b
end

f.separator = f:CreateTexture(nil, "ARTWORK")
f.separator:SetTexture("Interface\\Addons\\Sorted\\Textures\\Settings-Separator")
f.separator:SetPoint("TOPLEFT", f.left, "TOPRIGHT")
f.separator:SetPoint("BOTTOM")
f.separator:SetWidth(3)

-- ADD SETTINGS TO FRAME
f.right = CreateFrame("FRAME", nil, f)
f.right:SetPoint("TOPLEFT", f.left, "TOPRIGHT")
f.right:SetPoint("BOTTOMRIGHT")

for i,v in ipairs(settingsFrames) do
    f.left.AddButton(i, v.name)
    v:SetParent(f.right)
    v:SetAllPoints()
    v:SetShown(i == selectedTab)
    f.left.buttons[i].frame = v
end
f.left.UpdateButtons()